ForumName2 = LibStub("AceAddon-3.0"):NewAddon("ForumName2", "AceHook-3.0", "AceEvent-3.0", "AceConsole-3.0")
local string_gsub, string_format, string_match, string_lower, table_insert, table_wipe = string.gsub, string.format, string.match, string.lower, tinsert, wipe

local defaults = {
	profile = {
		customNotes = {}, 
	}
}

local chatEvents = {
	["CHAT_MSG_BATTLEGROUND"]   = true,
	["CHAT_MSG_CHANNEL"]        = true,
	["CHAT_MSG_GUILD"]          = true,
	["CHAT_MSG_OFFICER"]        = true,
	["CHAT_MSG_PARTY"]          = true,
	["CHAT_MSG_RAID"]           = true,
	["CHAT_MSG_RAID_LEADER"]    = true,
	["CHAT_MSG_RAID_WARNING"]   = true,
	["CHAT_MSG_SAY"]            = true,
	["CHAT_MSG_WHISPER"]        = true,
	["CHAT_MSG_WHISPER_INFORM"] = true,
	["CHAT_MSG_YELL"]           = true,
}
local guildNotes = {}
local systemMessageInfo = {
	{formatString = ERR_FRIEND_ONLINE_SS},
	{formatString = ERR_FRIEND_OFFLINE_S},
	{formatString = ERR_DECLINE_GROUP_S},
	{formatString = ERR_GUILD_DEMOTE_SSS},
	{formatString = ERR_GUILD_DISBAND_S},
	{formatString = ERR_GUILD_LEADER_CHANGED_SS},
	{formatString = ERR_GUILD_LEADER_S},
	{formatString = ERR_GUILD_LEAVE_S},
	{formatString = ERR_GUILD_PROMOTE_SSS},
	{formatString = ERR_GUILD_REMOVE_SS},
	{formatString = ERR_INVITED_TO_GROUP_SS},
	{formatString = ERR_INVITE_PLAYER_S},
	{formatString = ERR_JOINED_GROUP_S},
	{formatString = ERR_LEFT_GROUP_S},
	{formatString = ERR_NEW_LEADER_S},
	{formatString = ERR_NEW_LOOT_MASTER_S},
	{formatString = ERR_PLAYER_DIED_S},
	{formatString = ERR_RAID_MEMBER_ADDED_S},
	{formatString = ERR_RAID_MEMBER_REMOVED_S},
	{formatString = ERR_TRADE_BLOCKED_S},
	{formatString = ERR_TRADE_REQUEST_S}
}
local flags = {
	[1]  = {false, true},
	[4]  = {true, true, false},
	[6]  = {true, true},
	[9]  = {true, true, false},
	[10] = {true, true},
	[11] = {true, true}
}
local fillerTable = {true}
local existingNotes = {}
local selectedName, unitName, note

function ForumName2:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ForumNameDB", defaults, true)
end

function ForumName2:OnEnable()
	--self.db.profile.customNotes = {}
	for index, info in pairs(systemMessageInfo) do
		info.pattern = string_gsub(info.formatString, "%%s", "(%%a+)")
		info.pattern = string_gsub(info.pattern, "([%[%]])", "%%%1")
		info.replacementFlags = flags[index] or fillerTable
	end
	
	self:RegisterEvent("GUILD_ROSTER_UPDATE", "ScanRoster")
	self:RegisterEvent("PLAYER_GUILD_UPDATE", "PokeRoster")
	self:PokeRoster()
	
	for i = 1, NUM_CHAT_WINDOWS do
		self:RawHook(_G["ChatFrame" .. i], "AddMessage", "AddMessage", true)
	end
	
	for name, _ in pairs(self.db.profile.customNotes) do
		existingNotes[name] = name
	end
	
	local options = {
		type = "group", 
		args = {
			customNotes = {
				name = "Custom Notes", 
				type = "group", 
				guiInline = true, 
				order = 0, 
				args = {
					customNotesHavePriority = {
						name = "Prioritize custom notes", 
						desc = "Makes custom notes take priority over guild notes", 
						type = "toggle", 
						width = "full", 
						order = 0, 
						get = function() return self.db.profile.customNotesHavePriority end, 
						set = function(info, val) self.db.profile.customNotesHavePriority = val end
					},
					selectedName = {
						name = "Custom notes", 
						desc = "Existing custom notes", 
						type = "select", 
						values = existingNotes, 
						width = "full", 
						order = 1,
						get = function() return selectedName end,
						set = function(info, val)
							selectedName = val
							unitName = val
							note = self.db.profile.customNotes[val]
						end
					},
					unitName = {
						name = "Name", 
						desc = "Player name", 
						type = "input", 
						order = 2, 
						get = function() return unitName end,
						set = function(info, val) unitName = val end
					},
					note = {
						name = "Note", 
						desc = "Custom note", 
						type = "input", 
						order = 4, 
						get = function() return note end, 
						set = function(info, val) note = val end
					}, 
					addNote = {
						name = "Save Note", 
						desc = "Save Note", 
						type = "execute", 
						order = 6, 
						func = function()
							if(note and unitName and note ~= "" and unitName ~= "") then
								self.db.profile.customNotes[unitName] = note
								existingNotes[unitName] = unitName
								note = nil
								unitName = nil
							else
								self:Print("Unit name or note is missing or invalid")
							end
						end
					},
					deleteNote = {
						name = "Delete Note", 
						desc = "Delete Note", 
						type = "execute", 
						order = 8, 
						func = function()
							if(unitName and unitName ~= "") then
								local customNote = self.db.profile.customNotes[unitName]
								if(customNote) then
									self.db.profile.customNotes[unitName] = nil
									existingNotes[unitName] = nil
									note = nil
									unitName = nil
								else
									self:Print("Custom note for " .. unitName .. " not found")
								end
							else
								self:Print("Unit name is missing or invalid")
							end
						end
					},
				}
			},
		}
	}

	LibStub("AceConfig-3.0"):RegisterOptionsTable("ForumName2", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ForumName2", nil, nil, "customNotes")
	self:RegisterChatCommand("forumname", "ShowOptions")
	self:RegisterChatCommand("fn", "ShowOptions")
end

function ForumName2:ShowOptions()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end

function ForumName2:ScanRoster()
	self:UnregisterEvent("GUILD_ROSTER_UPDATE")
	if IsInGuild() then
		for i=1, GetNumGuildMembers(true) do
			local characterName, _, _, _, _, _, note = GetGuildRosterInfo(i)
			if(characterName and note) then
				guildNotes[characterName] = note
			end
		end
	end
	self:RegisterEvent("GUILD_ROSTER_UPDATE", "ScanRoster")
end

function ForumName2:PokeRoster()
	if IsInGuild() then
		GuildRoster()
	end
end

function ForumName2:GetNote(unitName)
	if(not unitName) then return nil end
	
	local source1, source2 = guildNotes, self.db.profile.customNotes
	if(self.db.profile.customNotesHavePriority) then
		source2 = guildNotes
		source1 = self.db.profile.customNotes
	end
	
	if(source1[unitName] or source1[string_lower(unitName)]) then 
		return source1[unitName] or source1[string_lower(unitName)] 
	end
	return source2[unitName] or source2[string_lower(unitName)]
end

local results = {}
function ForumName2:SystemMessageHelper(msg, info, ...)
	local eventFound, noteFound
	table_wipe(results)
	for i=1, select("#", ...) do
		eventFound = true
		local charName = select(i, ...)
		if(info.replacementFlags[i]) then
			local note = self:GetNote(charName)
			if(note) then
				table_insert(results, string_format("%s (%s)", charName, note))
				noteFound = true
			else
				table_insert(results, charName)
			end
		else
			table_insert(results, charName)
		end
	end
	if(noteFound) then
		return string_format(info.formatString, unpack(results))
	elseif(eventFound) then
		return msg
	end
end

function ForumName2:AddMessage(frame, msg, ...)
	if(type(msg) == "string") then
		for i, info in pairs(systemMessageInfo) do
			local output = self:SystemMessageHelper(msg, info, string_match(msg, info.pattern))
			if output then
				msg = output
				break
			end
		end
		
		if not output then
			local plink, pdata, pname = string_match(msg, "(|Hplayer:(.-)|h%[|c........(.-)|r%]|h)")
			if not plink then
				plink, pdata, pname = string_match(msg, "(|Hplayer:(.-)|h%[(.-)%]|h)")
			end
			if plink then
				local note = self:GetNote(pname)
				if note then
					msg = string_gsub(msg, "|h(.*)" .. pname .. "(.*)|h", "|h%1" .. pname .. " (" .. note .. ")" .. "%2|h", 1)
				end
			end
		end
	end
	self.hooks[frame]["AddMessage"](frame, msg, ...)
end
